import com.google.gson.Gson;
import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import org.apache.commons.io.IOUtils;

import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

public class WebhookHandler implements HttpHandler {
    public void handle (HttpExchange exchange) {
        String method = exchange.getRequestMethod();

        System.out.println("method: " + method);
        
        if (method.contains("GET")) {
            doGet(exchange);

        } else if (method.contains("POST")) {
            doPost(exchange);
        } else {
            System.out.println("method: " + method + "is illegal");
        }

    }

    private static Map<String,String> formData2Dic(String formData ) {
        Map<String,String> result = new HashMap<>();
        if(formData== null || formData.trim().length() == 0) {
            return result;
        }
        final String[] items = formData.split("&");
        Arrays.stream(items).forEach(item ->{
            final String[] keyAndVal = item.split("=");
            if( keyAndVal.length == 2) {
                try{
                    final String key = URLDecoder.decode( keyAndVal[0],"utf8");
                    final String val = URLDecoder.decode( keyAndVal[1],"utf8");
                    result.put(key,val);
                }catch (UnsupportedEncodingException e) {}
            }
        });
        return result;
    }

    private void doGet(HttpExchange exchange) {
        String queryString = exchange.getRequestURI().getQuery();
        Map<String, String> queryStringInfo = formData2Dic(queryString);
        String checkStr = queryStringInfo.get("check_str");
        Base64.Decoder decoder = Base64.getDecoder();
        String checkData = new String(decoder.decode(checkStr));
        // 由于获取checkStr时会把Base64默认补齐的"="去掉，这里再转一次补回来
        Base64.Encoder encoder = Base64.getEncoder();
        checkStr = encoder.encodeToString(checkData.getBytes());
        //System.out.println("check_str：" + checkStr + " checkData：" + checkData);

        String timestamp = exchange.getRequestHeaders().get("timestamp").get(0);
        String nonce = exchange.getRequestHeaders().get("nonce").get(0);
        String signature = exchange.getRequestHeaders().get("signature").get(0);
        //System.out.println("timestamp：" + timestamp + " nonce：" + nonce + " signature：" + signature);
        String checkSignature = Sha1Util.calSignature(WebhookConfig.token, timestamp, nonce, checkStr);
        //System.out.println("checkSignature：" + checkSignature);

        if (checkSignature.contentEquals(signature)) {
            try {
                // 验证通过时需要返回base64解码后的check_str参数，并且HTTP 头部响应200
                System.out.println(checkData);
                exchange.sendResponseHeaders(200, checkData.length());
                OutputStream os = exchange.getResponseBody();
                os.write(checkData.getBytes());
                os.close();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        } else {
            try {
                // 验证不通过时报错
                String response = "check signature failed";
                System.out.println(response);
                exchange.sendResponseHeaders(400, response.length());
                OutputStream os = exchange.getResponseBody();
                os.write(response.getBytes());
                os.close();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
    }

    private void doPost(HttpExchange exchange) {
        String timestamp = exchange.getRequestHeaders().get("timestamp").get(0);
        String nonce = exchange.getRequestHeaders().get("nonce").get(0);
        String signature = exchange.getRequestHeaders().get("signature").get(0);
        String postBody = null;
        PostMsg rawData = null;
        String msg = null;
        //System.out.println("timestamp：" + timestamp + " nonce：" + nonce + " signature：" + signature);
        try {
            postBody = IOUtils.toString(exchange.getRequestBody(), "UTF-8");
            //System.out.println(postBody + "\n");
            Gson gson = new Gson();
            rawData = gson.fromJson(postBody, PostMsg.class);
            //System.out.println(rawData.data);
            Base64.Decoder decoder = Base64.getDecoder();
            msg = new String(decoder.decode(rawData.data));
            //System.out.println(msg);
        } catch (Exception e) {
            System.out.println(e);
            throw new RuntimeException(e);
        }

        String checkSignature = Sha1Util.calSignature(WebhookConfig.token, timestamp, nonce, rawData.data);
        //System.out.println("checkSignature：" + checkSignature);

        if (checkSignature.contentEquals(signature)) {

            ////////////
            //对post消息进行处理，建议异步处理，防止超时
            System.out.println(msg);
            ////////////

            try {
                // 验证通过时固定的字符串，并且HTTP 头部响应200
                String response = "successfully received callback";
                System.out.println(response);
                exchange.sendResponseHeaders(200, response.length());
                OutputStream os = exchange.getResponseBody();
                os.write(response.getBytes());
                os.close();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        } else {
            try {
                // 验证不通过时报错
                String response = "check signature failed";
                System.out.println(response);
                exchange.sendResponseHeaders(400, response.length());
                OutputStream os = exchange.getResponseBody();
                os.write(response.getBytes());
                os.close();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
    }
}
